@@ -1,8 +1,8 @@
-## plot.phylo.R (2015-09-20)
+## plot.phylo.R (2016-07-01)
 
 ##   Plot Phylogenies
 
-## Copyright 2002-2015 Emmanuel Paradis
+## Copyright 2002-2016 Emmanuel Paradis
 
 ## This file is part of the R-package `ape'.
 ## See the file ../COPYING for licensing issues.
@@ -51,7 +51,17 @@ plot.phylo <-
                               "unrooted", "radial"))
     direction <- match.arg(direction, c("rightwards", "leftwards",
                                         "upwards", "downwards"))
-    if (is.null(x$edge.length)) use.edge.length <- FALSE
+    if (is.null(x$edge.length)) {
+        use.edge.length <- FALSE
+    } else {
+        if (use.edge.length && type != "radial") {
+            tmp <- sum(is.na(x$edge.length))
+            if (tmp) {
+                warning(paste(tmp, "branch length(s) NA(s): branch lengths ignored in the plot"))
+                use.edge.length <- FALSE
+            }
+        }
+    }
 
     if (is.numeric(align.tip.label)) {
         align.tip.label.lty <- align.tip.label
@@ -351,15 +361,21 @@ if (plot) {
     }
     if (root.edge) {
         rootcol <- if (length(edge.color) == 1) edge.color else "black"
+        rootw <- if (length(edge.width) == 1) edge.width else 1
+        rootlty <- if (length(edge.lty) == 1) edge.lty else 1
         if (type == "fan") {
             tmp <- polar2rect(x$root.edge, theta[ROOT])
-            segments(0, 0, tmp$x, tmp$y, col = rootcol)
+            segments(0, 0, tmp$x, tmp$y, col = rootcol, lwd = rootw, lty = rootlty)
         } else {
             switch(direction,
-                   "rightwards" = segments(0, yy[ROOT], x$root.edge, yy[ROOT], col = rootcol),
-                   "leftwards" = segments(xx[ROOT], yy[ROOT], xx[ROOT] + x$root.edge, yy[ROOT], col = rootcol),
-                   "upwards" = segments(xx[ROOT], 0, xx[ROOT], x$root.edge, col = rootcol),
-                   "downwards" = segments(xx[ROOT], yy[ROOT], xx[ROOT], yy[ROOT] + x$root.edge, col = rootcol))
+                   "rightwards" = segments(0, yy[ROOT], x$root.edge, yy[ROOT],
+                                           col = rootcol, lwd = rootw, lty = rootlty),
+                   "leftwards" = segments(xx[ROOT], yy[ROOT], xx[ROOT] + x$root.edge, yy[ROOT],
+                                          col = rootcol, lwd = rootw, lty = rootlty),
+                   "upwards" = segments(xx[ROOT], 0, xx[ROOT], x$root.edge,
+                                        col = rootcol, lwd = rootw, lty = rootlty),
+                   "downwards" = segments(xx[ROOT], yy[ROOT], xx[ROOT], yy[ROOT] + x$root.edge,
+                                          col = rootcol, lwd = rootw, lty = rootlty))
         }
     }
     if (show.tip.label) {
@@ -393,6 +409,13 @@ if (plot) {
                     if (type == "unrooted") "horizontal" else "axial"
                 } else match.arg(lab4ut, c("horizontal", "axial"))
 
+            xx.tips <- xx[1:Ntip]
+            yy.tips <- yy[1:Ntip]
+            if (label.offset) {
+                xx.tips <- xx.tips + label.offset * cos(angle)
+                yy.tips <- yy.tips + label.offset * sin(angle)
+            }
+
             if (lab4ut == "horizontal") {
                 y.adj <- x.adj <- numeric(Ntip)
                 sel <- abs(angle) > 0.75 * pi
@@ -403,12 +426,10 @@ if (plot) {
                 y.adj[sel] <- strheight(x$tip.label)[sel] / 2
                 sel <- angle < -pi / 4 & angle > -0.75 * pi
                 y.adj[sel] <- -strheight(x$tip.label)[sel] * 0.75
-                text(xx[1:Ntip] + x.adj * cex, yy[1:Ntip] + y.adj * cex,
+                text(xx.tips + x.adj * cex, yy.tips + y.adj * cex,
                      x$tip.label, adj = c(adj, 0), font = font,
                      srt = srt, cex = cex, col = tip.color)
             } else { # if lab4ut == "axial"
-                xx.tips <- xx[1:Ntip]
-                yy.tips <- yy[1:Ntip]
                 if (align.tip.label) {
                     POL <- rect2polar(xx.tips, yy.tips)
                     POL$r[] <- max(POL$r)
@@ -417,10 +438,6 @@ if (plot) {
                     yy.tips <- REC$y
                     segments(xx[1:Ntip], yy[1:Ntip], xx.tips, yy.tips, lty = align.tip.label.lty)
                 }
-                if (label.offset) {
-                    xx.tips <- xx.tips + label.offset * cos(angle)
-                    yy.tips <- yy.tips + label.offset * sin(angle)
-                }
                 if (type == "unrooted") {
                     adj <- abs(angle) > pi/2
                     angle <- angle * 180/pi # switch to degrees
